%%%% Anne Collins
%%%% LNCC, Brown University
%%%% 2012
%%%% Psych-review supplementary material
%%%% Cognitive control over learning: Creating, clustering and generalizing
%%%% Task-set structure


%%%% CTS model

function data = CTS(param,input,option)

alpha = param(1);% Dirichlet alpha param
beta = param(2);% softmax choice inverse temperature
tau = param(3);% volatility
b0 = param(4:6);% experts prior (C-TS, S-TS, flat)
init = param(7);% P(r|s,TS) initializing param
inits = param(8);% P(r|o) initializing param
epsilon = param(9);% TS noise param

belief = b0/sum(b0);

c1seq = input.cseq;% context for expert1
s2seq = c1seq;% stim for expert2
s1seq = input.sseq;% stimulus for expert2
c2seq = s1seq;% cont for expert 1
seq = input.seq;% flat sequence of stim
CAseq = input.CAseq;
noise = input.noise;
taille = length(seq);

nA = max(CAseq);% number actions
nS1 = max(c1seq);% number contexts (1)
nS2 = max(s1seq);% number stims (1)


% Initialize

Pco1=[zeros(nS1,1)];%P(TS|C)
Nrc1=init*ones(2,nS2,nA,1);%
Prc1=init*ones(2,nS2,nA,1)/(2*init);%P(r|s,TS)

Pco2=[zeros(nS2,1)];%Same, for other dim expert
Nrc2=init*ones(2,nS1,nA,1);%
Prc2=init*ones(2,nS1,nA,1)/(2*init);%


Nrcs = inits*ones(2,nS1*nS2,nA);
Prcs = Nrcs/(2*inits);% P(r|c,s)

o = seq(1);
ct1 = c1seq(1);
ct2 = s1seq(1);
Pco1(ct1) = 1;
Pco2(ct2) = 1;

% number of task-sets
nC1=1;
nC2=1;
% store interesting things
caPr = zeros(2,taille);% TS a priori
caPo = zeros(2,taille);% TS a posteriori
rseq = zeros(1,taille);% reward
prseq = zeros(1,taille);% p(correct action)
corseq = zeros(1,taille);% correct
Code = zeros(1,taille);% chosen action
bseq = zeros(2,taille);% expert weights

nCont1=0;
nCont2=0;
for i=1:taille
    o=seq(i);
    
    %%contexte et stim vus d'une dimension
    ct1 = c1seq(i);%contexte;
    st1 = s1seq(i);%stim;
    ct2=st1;
    st2=ct1;   
   
    %%Context-TS prior (exp1)
    if i~=1 && isempty(find(c1seq(1:i-1)==ct1))
        Pco1=[Pco1 zeros(nS1,1)];
        nCont1=nCont1+1;
        Pco1(ct1,1:nC1)=sum(Pco1(:,1:nC1))/(alpha+nCont1);
        Pco1(ct1,nC1+1)=alpha/(alpha+nCont1);%
        
        Nrc1(:,:,:,nC1+1)=init*ones(2,nS2,nA,1);
        Prc1(:,:,:,nC1+1)=ones(2,nS2,nA,1)/(2);%
        nC1=nC1+1;
    end
    PC1=Pco1(ct1,:);
    
    %%Context-TS prior (exp2)
    if i~=1 && isempty(find(c2seq(1:i-1)==ct2))
        Pco2=[Pco2 zeros(nS2,1)];
        nCont2=nCont2+1;
        Pco2(ct2,1:nC2)=sum(Pco2(:,1:nC2))/(alpha+nCont2);
        Pco2(ct2,nC2+1)=alpha/(alpha+nCont2);%
        
        Nrc2(:,:,:,nC2+1)=init*ones(2,nS1,nA,1);
        Prc2(:,:,:,nC2+1)=ones(2,nS1,nA,1)/(2);%
        nC2=nC2+1;
    end
    PC2=Pco2(ct2,:);
   
      
    %%Maximum a priori TS
    c1=find(PC1==max(PC1));
    if length(c1)>1
        tirage=randperm(length(c1));
        c1=c1(tirage(1));
    end
    c2=find(PC2==max(PC2));
    if length(c2)>1
        tirage=randperm(length(c2));
        c2=c2(tirage(1));
    end
    
    caPr(:,i) = [c1 ; c2];
    cQ1=c1;
    cQ2=c2;
    % Q values
    Q1 = Prc1(2,st1,:,cQ1);
    Q2 = Prc2(2,st2,:,cQ2);
    Q3 = Prcs(2,o,:);
    %within TS noise
    S1eps = unique(s1seq(1:i-1));
    S2eps = unique(s2seq(1:i-1));
    Seps = unique(seq(1:i-1));
    Q1e = mean(Prc1(2,S1eps,:,cQ1),2);
    Q2e = mean(Prc2(2,S2eps,:,cQ2),2);
    Q3e = mean(Prcs(2,Seps,:),2);
    
    %within TS noise
    pr1e=exp(beta*squeeze(Q1e));pr1e=pr1e/sum(pr1e);
    pr2e=exp(beta*squeeze(Q2e));pr2e=pr2e/sum(pr2e);
    pr3e=exp(beta*squeeze(Q3e));pr3e=pr3e/sum(pr3e);
    
    %softmax action choice
    pr1 = exp(beta*squeeze(Q1));
    pr1 = (1-epsilon)*pr1/sum(pr1) + epsilon*pr1e;
    pr2 = exp(beta*squeeze(Q2));
    pr2 = (1-epsilon)*pr2/sum(pr2) + epsilon*pr2e;
    pr3 = exp(beta*squeeze(Q3));
    pr3 = (1-epsilon)*pr3/sum(pr3) + epsilon*pr3e;
    
    bseq(:,i)=[belief(1) ;belief(2)];
    
    % combine probabilities
    pr=belief(1)*pr1+belief(2)*pr2+belief(3)*pr3;
    
    % pick action
    ra=rand;
    CDP = [0 cumsum(pr')];
    choice = find(CDP<ra);
    a = choice(end);
    
    prseq(i) = pr(CAseq(i));
    
    %get reward, correct
    if a == CAseq(i); cor = 1; else cor = 0; end
    if rand < noise; r = 1-cor; else r = cor; end

    corseq(i) = cor;
    rseq(i) = r;
    Code(i) = a;
    
    % Posterior
    PC1 = PC1.*reshape(Prc1(r+1,st1,a,:),1,nC1);
    PC2 = PC2.*reshape(Prc2(r+1,st2,a,:),1,nC2);
    PC1 = PC1/sum(PC1);
    PC2 = PC2/sum(PC2);
   

    % a posteriori TS
    cc1 = find(PC1==max(PC1));%%deduction de l'etat cache actuel a posteriori
    if length(cc1)>1;        cc1=c1;end
    c1 = cc1;
    cc2 = find(PC2==max(PC2));%%deduction de l'etat cache actuel a posteriori
    if length(cc2)>1;cc2=c2;end
    c2 = cc2;

    caPo(:,i) = [c1;c2];

    %learning
    %hier expert 1
    b1 = Prc1(r+1,st1,a,cQ1);
    Pco1(ct1,1:nC1) = PC1(1:nC1);
    Nrc1(r+1,st1,a,c1) = Nrc1(r+1,st1,a,c1)+belief(1);%%1;%%
    Prc1(:,st1,a,c1) = Nrc1(:,st1,a,c1)/sum(Nrc1(:,st1,a,c1));
    %hier expert 2
    b2 = Prc2(r+1,st2,a,cQ2);
    Pco2(ct2,1:nC2) = PC2(1:nC2);
    Nrc2(r+1,st2,a,c2) = Nrc2(r+1,st2,a,c2)+belief(2);%%1;%%
    Prc2(:,st2,a,c2) = Nrc2(:,st2,a,c2)/sum(Nrc2(:,st2,a,c2));
    %flat expert
    b3 = Prcs(r+1,o,a);
    Nrcs(r+1,o,a) = Nrcs(r+1,o,a)+belief(3);%%1;%%
    Prcs(:,o,a) = Nrcs(:,o,a)/sum(Nrcs(:,o,a));
    % update confidence in experts
    belief = [b1 b2 b3].*belief;
    belief = belief/sum(belief);
    belief = tau*belief+(1-tau)*b0/sum(b0);

    
end


data = [];
data.bseq = bseq;
N = max(Pco1(:,:));
N = sum(N);
data.N = N;
data.Input = seq;
data.Cseq = c1seq;
data.Sseq = s1seq;
data.Code = Code;
data.corseq = corseq;
data.rseq = rseq;
data.caPr = caPr;
data.caPo = caPo;


end 